@@ -6,30 +6,16 @@ import ntpath
 import os.path
 import posixpath
 import re
-import warnings
 from pathlib import Path
 from typing import List, Optional, Pattern, Tuple
 
-from ._base import DEFAULT_MIN_LEN, AbstractSanitizer, BaseFile, BaseValidator
-from ._common import (
-    PathType,
-    Platform,
-    PlatformType,
-    findall_to_str,
-    is_pathlike_obj,
-    preprocess,
-    validate_pathtype,
-)
-from ._const import _NTFS_RESERVED_FILE_NAMES
+from ._base import AbstractSanitizer, AbstractValidator, BaseFile, BaseValidator
+from ._common import findall_to_str, to_str, validate_pathtype
+from ._const import _NTFS_RESERVED_FILE_NAMES, DEFAULT_MIN_LEN, INVALID_CHAR_ERR_MSG_TMPL, Platform
 from ._filename import FileNameSanitizer, FileNameValidator
-from .error import (
-    ErrorReason,
-    InvalidCharError,
-    InvalidLengthError,
-    ReservedNameError,
-    ValidationError,
-)
-from .handler import Handler
+from ._types import PathType, PlatformType
+from .error import ErrorReason, InvalidCharError, ReservedNameError, ValidationError
+from .handler import NullValueHandler
 
 
 _RE_INVALID_PATH = re.compile(f"[{re.escape(BaseFile._INVALID_PATH_CHARS):s}]", re.UNICODE)
@@ -39,62 +25,69 @@ _RE_INVALID_WIN_PATH = re.compile(f"[{re.escape(BaseFile._INVALID_WIN_PATH_CHARS
 class FilePathSanitizer(AbstractSanitizer):
     def __init__(
         self,
-        min_len: int = DEFAULT_MIN_LEN,
         max_len: int = -1,
-        platform: PlatformType = None,
+        fs_encoding: Optional[str] = None,
+        platform: Optional[PlatformType] = None,
         check_reserved: bool = True,
-        null_value_handler: Optional[Handler] = None,
+        null_value_handler: Optional[NullValueHandler] = None,
         normalize: bool = True,
+        validate_after_sanitize: bool = False,
+        validator: Optional[AbstractValidator] = None,
     ) -> None:
+        if validator:
+            fpath_validator = validator
+        else:
+            fpath_validator = FilePathValidator(
+                min_len=DEFAULT_MIN_LEN,
+                max_len=max_len,
+                fs_encoding=fs_encoding,
+                check_reserved=check_reserved,
+                platform=platform,
+            )
         super().__init__(
-            min_len=min_len,
             max_len=max_len,
+            fs_encoding=fs_encoding,
             check_reserved=check_reserved,
+            validator=fpath_validator,
             null_value_handler=null_value_handler,
             platform=platform,
+            validate_after_sanitize=validate_after_sanitize,
         )
 
         self._sanitize_regexp = self._get_sanitize_regexp()
-        self.__fpath_validator = FilePathValidator(
-            min_len=self.min_len,
-            max_len=self.max_len,
-            check_reserved=check_reserved,
-            platform=self.platform,
-        )
         self.__fname_sanitizer = FileNameSanitizer(
-            min_len=self.min_len,
             max_len=self.max_len,
+            fs_encoding=fs_encoding,
             check_reserved=check_reserved,
+            null_value_handler=null_value_handler,
             platform=self.platform,
+            validate_after_sanitize=validate_after_sanitize,
         )
         self.__normalize = normalize
 
-        if self._is_universal() or self._is_windows():
+        if self._is_windows(include_universal=True):
             self.__split_drive = ntpath.splitdrive
         else:
             self.__split_drive = posixpath.splitdrive
 
     def sanitize(self, value: PathType, replacement_text: str = "") -> PathType:
         try:
-            validate_pathtype(value, allow_whitespaces=False if self._is_windows() else True)
+            validate_pathtype(value, allow_whitespaces=not self._is_windows(include_universal=True))
         except ValidationError as e:
             if e.reason == ErrorReason.NULL_NAME:
+                if isinstance(value, Path):
+                    raise
+
                 return self._null_value_handler(e)
             raise
 
-        self.__fpath_validator.validate_abspath(value)
-
-        unicode_filepath = preprocess(value)
-
-        if self.__normalize:
-            unicode_filepath = os.path.normpath(unicode_filepath)
+        unicode_filepath = to_str(value)
 
         drive, unicode_filepath = self.__split_drive(unicode_filepath)
-        sanitized_path = self._sanitize_regexp.sub(replacement_text, unicode_filepath)
-        if self._is_windows():
-            path_separator = "\\"
-        else:
-            path_separator = "/"
+        unicode_filepath = self._sanitize_regexp.sub(replacement_text, unicode_filepath)
+        if self.__normalize and unicode_filepath:
+            unicode_filepath = os.path.normpath(unicode_filepath)
+        sanitized_path = unicode_filepath
 
         sanitized_entries: List[str] = []
         if drive:
@@ -104,7 +97,9 @@ class FilePathSanitizer(AbstractSanitizer):
                 sanitized_entries.append(f"{entry}_")
                 continue
 
-            sanitized_entry = str(self.__fname_sanitizer.sanitize(entry))
+            sanitized_entry = str(
+                self.__fname_sanitizer.sanitize(entry, replacement_text=replacement_text)
+            )
             if not sanitized_entry:
                 if not sanitized_entries:
                     sanitized_entries.append("")
@@ -112,26 +107,33 @@ class FilePathSanitizer(AbstractSanitizer):
 
             sanitized_entries.append(sanitized_entry)
 
-        sanitized_path = path_separator.join(sanitized_entries)
+        sanitized_path = self.__get_path_separator().join(sanitized_entries)
         try:
-            self.__fpath_validator.validate(sanitized_path)
+            self._validator.validate(sanitized_path)
         except ValidationError as e:
             if e.reason == ErrorReason.NULL_NAME:
-                return self._null_value_handler(e)
+                sanitized_path = self._null_value_handler(e)
 
-            raise
+        if self._validate_after_sanitize:
+            self._validator.validate(sanitized_path)
 
-        if is_pathlike_obj(value):
+        if isinstance(value, Path):
             return Path(sanitized_path)
 
         return sanitized_path
 
-    def _get_sanitize_regexp(self) -> Pattern:
-        if self.platform in [Platform.UNIVERSAL, Platform.WINDOWS]:
+    def _get_sanitize_regexp(self) -> Pattern[str]:
+        if self._is_windows(include_universal=True):
             return _RE_INVALID_WIN_PATH
 
         return _RE_INVALID_PATH
 
+    def __get_path_separator(self) -> str:
+        if self._is_windows():
+            return "\\"
+
+        return "/"
+
 
 class FilePathValidator(BaseValidator):
     _RE_NTFS_RESERVED = re.compile(
@@ -156,12 +158,14 @@ class FilePathValidator(BaseValidator):
         self,
         min_len: int = DEFAULT_MIN_LEN,
         max_len: int = -1,
-        platform: PlatformType = None,
+        fs_encoding: Optional[str] = None,
+        platform: Optional[PlatformType] = None,
         check_reserved: bool = True,
     ) -> None:
         super().__init__(
             min_len=min_len,
             max_len=max_len,
+            fs_encoding=fs_encoding,
             check_reserved=check_reserved,
             platform=platform,
         )
@@ -170,37 +174,42 @@ class FilePathValidator(BaseValidator):
             min_len=min_len, max_len=max_len, check_reserved=check_reserved, platform=platform
         )
 
-        if self._is_universal() or self._is_windows():
+        if self._is_windows(include_universal=True):
             self.__split_drive = ntpath.splitdrive
         else:
             self.__split_drive = posixpath.splitdrive
 
     def validate(self, value: PathType) -> None:
-        validate_pathtype(
-            value,
-            allow_whitespaces=False
-            if self.platform in [Platform.UNIVERSAL, Platform.WINDOWS]
-            else True,
-        )
+        validate_pathtype(value, allow_whitespaces=not self._is_windows(include_universal=True))
         self.validate_abspath(value)
 
-        _drive, value = self.__split_drive(str(value))
-        if not value:
+        _drive, tail = self.__split_drive(value)
+        if not tail:
             return
 
-        filepath = os.path.normpath(value)
-        unicode_filepath = preprocess(filepath)
-        value_len = len(unicode_filepath)
+        unicode_filepath = to_str(tail)
+        byte_ct = len(unicode_filepath.encode(self._fs_encoding))
+        err_kwargs = {
+            "reason": ErrorReason.INVALID_LENGTH,
+            "platform": self.platform,
+            "fs_encoding": self._fs_encoding,
+        }
 
-        if value_len > self.max_len:
-            raise InvalidLengthError(
-                f"file path is too long: expected<={self.max_len:d}, actual={value_len:d}"
+        if byte_ct > self.max_len:
+            raise ValidationError(
+                [
+                    f"file path is too long: expected<={self.max_len:d} bytes, actual={byte_ct:d} bytes"
+                ],
+                **err_kwargs,
             )
-        if value_len < self.min_len:
-            raise InvalidLengthError(
-                "file path is too short: expected>={:d}, actual={:d}".format(
-                    self.min_len, value_len
-                )
+        if byte_ct < self.min_len:
+            raise ValidationError(
+                [
+                    "file path is too short: expected>={:d} bytes, actual={:d} bytes".format(
+                        self.min_len, byte_ct
+                    )
+                ],
+                **err_kwargs,
             )
 
         self._validate_reserved_keywords(unicode_filepath)
@@ -211,13 +220,12 @@ class FilePathValidator(BaseValidator):
 
             self.__fname_validator._validate_reserved_keywords(entry)
 
-        if self._is_universal() or self._is_windows():
+        if self._is_windows(include_universal=True):
             self.__validate_win_filepath(unicode_filepath)
         else:
             self.__validate_unix_filepath(unicode_filepath)
 
     def validate_abspath(self, value: PathType) -> None:
-        value = str(value)
         is_posix_abs = posixpath.isabs(value)
         is_nt_abs = ntpath.isabs(value)
         err_object = ValidationError(
@@ -225,8 +233,8 @@ class FilePathValidator(BaseValidator):
                 "an invalid absolute file path ({}) for the platform ({}).".format(
                     value, self.platform.value
                 )
-                + " to avoid the error, specify an appropriate platform correspond"
-                + " with the path format, or 'auto'."
+                + " to avoid the error, specify an appropriate platform corresponding to"
+                + " the path format or 'auto'."
             ),
             platform=self.platform,
             reason=ErrorReason.MALFORMED_ABS_PATH,
@@ -238,17 +246,14 @@ class FilePathValidator(BaseValidator):
         if self._is_universal() and any([is_posix_abs, is_nt_abs]):
             ValidationError(
                 description=(
-                    "{}. expected a platform independent file path".format(
-                        "POSIX absolute file path found"
-                        if is_posix_abs
-                        else "NT absolute file path found"
-                    )
+                    ("POSIX style" if is_posix_abs else "NT style")
+                    + " absolute file path found. expected a platform-independent file path."
                 ),
                 platform=self.platform,
                 reason=ErrorReason.MALFORMED_ABS_PATH,
             )
 
-        if any([self._is_windows(), self._is_universal()]) and is_posix_abs:
+        if self._is_windows(include_universal=True) and is_posix_abs:
             raise err_object
 
         drive, _tail = ntpath.splitdrive(value)
@@ -259,7 +264,7 @@ class FilePathValidator(BaseValidator):
         match = _RE_INVALID_PATH.findall(unicode_filepath)
         if match:
             raise InvalidCharError(
-                self._ERROR_MSG_TEMPLATE.format(
+                INVALID_CHAR_ERR_MSG_TMPL.format(
                     invalid=findall_to_str(match), value=repr(unicode_filepath)
                 )
             )
@@ -268,7 +273,7 @@ class FilePathValidator(BaseValidator):
         match = _RE_INVALID_WIN_PATH.findall(unicode_filepath)
         if match:
             raise InvalidCharError(
-                self._ERROR_MSG_TEMPLATE.format(
+                INVALID_CHAR_ERR_MSG_TMPL.format(
                     invalid=findall_to_str(match), value=repr(unicode_filepath)
                 ),
                 platform=Platform.WINDOWS,
@@ -289,9 +294,10 @@ class FilePathValidator(BaseValidator):
 
 def validate_filepath(
     file_path: PathType,
-    platform: Optional[str] = None,
+    platform: Optional[PlatformType] = None,
     min_len: int = DEFAULT_MIN_LEN,
     max_len: Optional[int] = None,
+    fs_encoding: Optional[str] = None,
     check_reserved: bool = True,
 ) -> None:
     """Verifying whether the ``file_path`` is a valid file path or not.
@@ -304,16 +310,19 @@ def validate_filepath(
 
             .. include:: platform.txt
         min_len:
-            Minimum length of the ``file_path``. The value must be greater or equal to one.
+            Minimum byte length of the ``file_path``. The value must be greater or equal to one.
             Defaults to ``1``.
         max_len:
-            Maximum length of the ``file_path`` length. If the value is |None| or minus,
+            Maximum byte length of the ``file_path``. If the value is |None| or minus,
             automatically determined by the ``platform``:
 
                 - ``Linux``: 4096
                 - ``macOS``: 1024
                 - ``Windows``: 260
                 - ``universal``: 260
+        fs_encoding:
+            Filesystem encoding that used to calculate the byte length of the file path.
+            If |None|, get the value from the execution environment.
         check_reserved:
             If |True|, check reserved names of the ``platform``.
 
@@ -321,7 +330,7 @@ def validate_filepath(
         ValidationError (ErrorReason.INVALID_CHARACTER):
             If the ``file_path`` includes invalid char(s):
             |invalid_file_path_chars|.
-            The following characters are also invalid for Windows platform:
+            The following characters are also invalid for Windows platforms:
             |invalid_win_file_path_chars|
         ValidationError (ErrorReason.INVALID_LENGTH):
             If the ``file_path`` is longer than ``max_len`` characters.
@@ -340,21 +349,17 @@ def validate_filepath(
         platform=platform,
         min_len=min_len,
         max_len=-1 if max_len is None else max_len,
+        fs_encoding=fs_encoding,
         check_reserved=check_reserved,
     ).validate(file_path)
 
 
-def validate_file_path(file_path, platform=None, max_path_len=None):
-    warnings.warn("'validate_file_path' has moved to 'validate_filepath'", DeprecationWarning)
-
-    validate_filepath(file_path, platform, max_path_len)
-
-
 def is_valid_filepath(
     file_path: PathType,
-    platform: Optional[str] = None,
+    platform: Optional[PlatformType] = None,
     min_len: int = DEFAULT_MIN_LEN,
     max_len: Optional[int] = None,
+    fs_encoding: Optional[str] = None,
     check_reserved: bool = True,
 ) -> bool:
     """Check whether the ``file_path`` is a valid name or not.
@@ -374,6 +379,7 @@ def is_valid_filepath(
         platform=platform,
         min_len=min_len,
         max_len=-1 if max_len is None else max_len,
+        fs_encoding=fs_encoding,
         check_reserved=check_reserved,
     ).is_valid(file_path)
 
@@ -381,15 +387,17 @@ def is_valid_filepath(
 def sanitize_filepath(
     file_path: PathType,
     replacement_text: str = "",
-    platform: Optional[str] = None,
+    platform: Optional[PlatformType] = None,
     max_len: Optional[int] = None,
+    fs_encoding: Optional[str] = None,
     check_reserved: bool = True,
-    null_value_handler: Optional[Handler] = None,
+    null_value_handler: Optional[NullValueHandler] = None,
     normalize: bool = True,
+    validate_after_sanitize: bool = False,
 ) -> PathType:
     """Make a valid file path from a string.
 
-    To make a valid file path the function does:
+    To make a valid file path, the function does the following:
 
         - replace invalid characters for a file path within the ``file_path``
           with the ``replacement_text``. Invalid characters are as follows:
@@ -413,14 +421,17 @@ def sanitize_filepath(
 
             .. include:: platform.txt
         max_len:
-            Maximum length of the ``file_path`` length. Truncate the name if the ``file_path``
-            length exceedd this value. If the value is |None| or minus,
-            ``max_len`` will automatically determined by the ``platform``:
+            Maximum byte length of the file path.
+            Truncate the path if the value length exceeds the `max_len`.
+            If the value is |None| or minus, ``max_len`` will automatically determined by the ``platform``:
 
                 - ``Linux``: 4096
                 - ``macOS``: 1024
                 - ``Windows``: 260
                 - ``universal``: 260
+        fs_encoding:
+            Filesystem encoding that used to calculate the byte length of the file path.
+            If |None|, get the value from the execution environment.
         check_reserved:
             If |True|, sanitize reserved names of the ``platform``.
         null_value_handler:
@@ -428,6 +439,8 @@ def sanitize_filepath(
             Defaults to ``pathvalidate.handler.return_null_string()`` that just return ``""``.
         normalize:
             If |True|, normalize the the file path.
+        validate_after_sanitize:
+            Execute validation after sanitization to the file path.
 
     Returns:
         Same type as the argument (str or PathLike object):
@@ -444,13 +457,9 @@ def sanitize_filepath(
     return FilePathSanitizer(
         platform=platform,
         max_len=-1 if max_len is None else max_len,
+        fs_encoding=fs_encoding,
         check_reserved=check_reserved,
         normalize=normalize,
         null_value_handler=null_value_handler,
+        validate_after_sanitize=validate_after_sanitize,
     ).sanitize(file_path, replacement_text)
-
-
-def sanitize_file_path(file_path, replacement_text="", platform=None, max_path_len=None):
-    warnings.warn("'sanitize_file_path' has moved to 'sanitize_filepath'", DeprecationWarning)
-
-    return sanitize_filepath(file_path, platform, max_path_len)
