@@ -4,6 +4,7 @@
 
 import platform as m_platform
 import random
+import sys
 from collections import OrderedDict
 from itertools import chain, product
 from pathlib import Path
@@ -19,8 +20,9 @@ from pathvalidate import (
     sanitize_filename,
     validate_filename,
 )
-from pathvalidate._common import is_pathlike_obj, unprintable_ascii_chars
+from pathvalidate._common import unprintable_ascii_chars
 from pathvalidate._filename import FileNameSanitizer, FileNameValidator
+from pathvalidate.handler import raise_error, return_null_string, return_timestamp
 
 from ._common import (
     INVALID_FILENAME_CHARS,
@@ -47,7 +49,11 @@ VALID_MULTIBYTE_NAMES = ["新しいテキスト ドキュメント.txt", "新規
 class Test_FileSanitizer:
     @pytest.mark.parametrize(
         ["test_platform", "expected"],
-        [["windows", Platform.WINDOWS], ["linux", Platform.LINUX], ["macos", Platform.MACOS]],
+        [
+            ["windows", Platform.WINDOWS],
+            ["linux", Platform.LINUX],
+            ["macos", Platform.MACOS],
+        ],
     )
     def test_normal_platform_auto(self, monkeypatch, test_platform, expected):
         if test_platform == "windows":
@@ -57,12 +63,14 @@ class Test_FileSanitizer:
         elif test_platform == "macos":
             patch = lambda: "macos"
         else:
-            raise ValueError("unexpected test platform: {}".format(test_platform))
+            raise ValueError(f"unexpected test platform: {test_platform}")
 
         monkeypatch.setattr(m_platform, "system", patch)
 
         assert FileNameSanitizer(255, platform="auto").platform == expected
 
+
+class Test_FileNameValidator:
     @pytest.mark.parametrize(
         ["test_platform", "expected"],
         [
@@ -130,7 +138,11 @@ class Test_validate_filename:
 
     @pytest.mark.parametrize(
         ["platform"],
-        [["linux"], ["macos"], ["posix"]],
+        [
+            ["linux"],
+            ["macos"],
+            ["posix"],
+        ],
     )
     def test_normal_only_whitespaces(self, platform):
         value = "  "
@@ -139,7 +151,10 @@ class Test_validate_filename:
 
     @pytest.mark.parametrize(
         ["platform"],
-        [["windows"], ["universal"]],
+        [
+            ["windows"],
+            ["universal"],
+        ],
     )
     def test_abnormal_only_whitespaces(self, platform):
         value = "  "
@@ -181,7 +196,7 @@ class Test_validate_filename:
     @pytest.mark.parametrize(
         ["value", "platform", "max_len", "expected"],
         [
-            ["a" * 255, None, None, None],
+            ["a" * 255, None, -1, None],
             ["a" * 5000, None, 10000, ErrorReason.INVALID_LENGTH],
             ["valid_length", "universal", 255, None],
             ["valid_length", Platform.UNIVERSAL, 255, None],
@@ -199,18 +214,35 @@ class Test_validate_filename:
         assert e.value.reason == expected
 
     @pytest.mark.parametrize(
-        ["value", "platform", "max_len", "expected"],
+        ["value", "platform", "fs_encoding", "max_len", "expected"],
         [
-            ["invalid_max_len", None, 0, ValueError],
+            ["あ" * 85, "universal", "utf-8", 255, None],
+            ["あ" * 86, "universal", "utf-8", 255, ErrorReason.INVALID_LENGTH],
+            ["あ" * 126, "universal", "utf-16", 255, None],
+            ["あ" * 127, "universal", "utf-16", 255, ErrorReason.INVALID_LENGTH],
         ],
     )
-    def test_abnormal_max_len(self, value, platform, max_len, expected):
-        with pytest.raises(expected):
-            validate_filename(value, platform=platform, max_len=max_len)
+    def test_max_len_fs_encoding(self, value, platform, fs_encoding, max_len, expected):
+        kwargs = {
+            "platform": platform,
+            "max_len": max_len,
+            "fs_encoding": fs_encoding,
+        }
+
+        if expected is None:
+            validate_filename(value, **kwargs)
+            assert is_valid_filename(value, **kwargs)
+            return
+
+        with pytest.raises(ValidationError) as e:
+            validate_filename(value, **kwargs)
+        assert e.value.reason == expected
 
     @pytest.mark.parametrize(
         ["value", "min_len", "max_len", "expected"],
         [
+            ["minux max_len", 1, -1, None],
+            ["zero max_len", 1, 0, None],
             ["valid length", 1, 255, None],
             ["eq min max", 10, 10, None],
             ["inversion", 100, 1, ValueError],
@@ -283,7 +315,7 @@ class Test_validate_filename:
             )
         ]
         + [
-            ["{}.txt".format(reserved_keyword), platform, ValidationError]
+            [f"{reserved_keyword}.txt", platform, ValidationError]
             for reserved_keyword, platform in product(
                 WIN_RESERVED_FILE_NAMES, ["windows", "universal"]
             )
@@ -362,8 +394,8 @@ class Test_validate_filename:
 
         assert e.value.reason == ErrorReason.INVALID_CHARACTER
         assert str(e.value) == (
-            r"invalid char found: invalids=('\r'), value='asdf\rsdf', "
-            "reason=INVALID_CHARACTER, target-platform=Windows"
+            r"[PV1100] invalid characters found: invalids=('\r'), value='asdf\rsdf', "
+            "target-platform=Windows"
         )  # noqa
 
     @pytest.mark.parametrize(
@@ -371,6 +403,16 @@ class Test_validate_filename:
         [
             [None, ValueError],
             ["", ValidationError],
+        ],
+    )
+    def test_exception_null_value(self, value, expected):
+        with pytest.raises(expected):
+            validate_filename(value)
+        assert not is_valid_filename(value)
+
+    @pytest.mark.parametrize(
+        ["value", "expected"],
+        [
             ["a" * 256, ValidationError],
             [1, TypeError],
             [True, TypeError],
@@ -406,7 +448,14 @@ class Test_sanitize_filename:
             for pair in AllPairs(
                 OrderedDict(
                     {
-                        "platform": ["posix", "linux", "macos"],
+                        "platform": [
+                            "posix",
+                            "linux",
+                            "macos",
+                            Platform.POSIX,
+                            Platform.LINUX,
+                            Platform.MACOS,
+                        ],
                         "c": INVALID_PATH_CHARS + unprintable_ascii_chars,
                         "repl": REPLACE_TEXT_LIST,
                     }
@@ -418,7 +467,14 @@ class Test_sanitize_filename:
             for pair in AllPairs(
                 OrderedDict(
                     {
-                        "platform": ["posix", "linux", "macos"],
+                        "platform": [
+                            "posix",
+                            "linux",
+                            "macos",
+                            Platform.POSIX,
+                            Platform.LINUX,
+                            Platform.MACOS,
+                        ],
                         "c": [":", "*", "?", '"', "<", ">", "|"],
                         "repl": REPLACE_TEXT_LIST,
                     }
@@ -447,24 +503,33 @@ class Test_sanitize_filename:
     def test_normal_pathlike(self, value, replace_text, expected):
         sanitized_name = sanitize_filename(value, replace_text)
         assert sanitized_name == expected
-        assert is_pathlike_obj(sanitized_name)
+        assert isinstance(sanitized_name, Path)
 
         validate_filename(sanitized_name)
         assert is_valid_filename(sanitized_name)
 
     @pytest.mark.parametrize(
-        ["value", "expected"],
+        ["value"],
         [
-            ["", ""],
-            [None, ""],
+            [None],
+            [""],
+            ["/"],
+            ["//"],
+            ["?"],
         ],
     )
-    def test_normal_null_values(self, value, expected):
-        assert sanitize_filename(value) == expected
+    def test_normal_null_value_handler(self, value):
+        assert sanitize_filename(value, null_value_handler=return_null_string) == ""
+        assert sanitize_filename(value, null_value_handler=return_timestamp) != ""
+        with pytest.raises(ValidationError):
+            sanitize_filename(value, null_value_handler=raise_error)
 
     @pytest.mark.parametrize(
         ["value", "replace_text", "expected"],
-        [["あい/うえお.txt", "", "あいうえお.txt"], ["属/性.txt", "-", "属-性.txt"]],
+        [
+            ["あい/うえお.txt", "", "あいうえお.txt"],
+            ["属/性.txt", "-", "属-性.txt"],
+        ],
     )
     def test_normal_multibyte(self, value, replace_text, expected):
         sanitized_name = sanitize_filename(value, replace_text)
@@ -474,7 +539,11 @@ class Test_sanitize_filename:
 
     @pytest.mark.parametrize(
         ["value", "max_len", "expected"],
-        [["a" * 10, 255, 10], ["invalid_length" * 100, 255, 255], ["invalid_length" * 100, 10, 10]],
+        [
+            ["a" * 10, 255, 10],
+            ["invalid_length" * 100, 255, 255],
+            ["invalid_length" * 100, 10, 10],
+        ],
     )
     def test_normal_max_len(self, value, max_len, expected):
         filename = sanitize_filename(value, max_len=max_len)
@@ -488,7 +557,7 @@ class Test_sanitize_filename:
             for reserved in WIN_RESERVED_FILE_NAMES
         ]
         + [
-            ["{}.txt".format(reserved_keyword), platform, "{}_.txt".format(reserved_keyword)]
+            [f"{reserved_keyword}.txt", platform, f"{reserved_keyword}_.txt"]
             for reserved_keyword, platform in product(
                 WIN_RESERVED_FILE_NAMES, ["windows", "universal"]
             )
@@ -516,23 +585,36 @@ class Test_sanitize_filename:
         ],
     )
     def test_normal_check_reserved(self, value, check_reserved, expected):
-        assert (
-            sanitize_filename(value, platform="windows", check_reserved=check_reserved) == expected
-        )
+        for platform in ["windows", "universal"]:
+            assert (
+                sanitize_filename(value, platform=platform, check_reserved=check_reserved)
+                == expected
+            )
 
     @pytest.mark.parametrize(
         ["platform", "value", "expected"],
         [
             ["windows", "period.", "period"],
             ["windows", "space ", "space"],
+            ["windows", " space ", "space"],
             ["windows", "space_and_period .", "space_and_period"],
             ["windows", "space_and_period. ", "space_and_period"],
+            ["windows", " .space_and_period", ".space_and_period"],
+            ["windows", ". space_and_period", ". space_and_period"],
+            ["windows", ". ", "."],
+            ["windows", " .", "."],
+            ["windows", " . ", "."],
+            ["windows", ".. ", ".."],
             ["linux", "period.", "period."],
             ["linux", "space ", "space "],
             ["linux", "space_and_period. ", "space_and_period. "],
             ["universal", "period.", "period"],
             ["universal", "space ", "space"],
             ["universal", "space_and_period .", "space_and_period"],
+            ["universal", ". ", "."],
+            ["universal", " .", "."],
+            ["universal", " . ", "."],
+            ["universal", ".. ", ".."],
         ],
     )
     def test_normal_space_or_period_at_tail(self, platform, value, expected):
@@ -540,7 +622,38 @@ class Test_sanitize_filename:
         assert filename == expected
         assert is_valid_filename(filename, platform=platform)
 
-    @pytest.mark.parametrize(["value", "expected"], [[1, TypeError], [True, TypeError]])
+    @pytest.mark.parametrize(
+        ["platform", "value"],
+        [
+            [platform, value]
+            for platform, value in product(
+                ["windows", "universal"],
+                [
+                    "\a\r",
+                ],
+            )
+        ],
+    )
+    def test_exception_invalid_after_sanitize(self, platform, value):
+        kwargs = {
+            "platform": platform,
+            "replacement_text": "",
+            "validate_after_sanitize": False,
+        }
+        print("'{}'".format(sanitize_filename(value, **kwargs)), file=sys.stderr)
+        kwargs["validate_after_sanitize"] = True
+
+        with pytest.raises(ValidationError) as e:
+            sanitize_filename(value, **kwargs)
+        assert e.value.reason == ErrorReason.INVALID_AFTER_SANITIZE
+
+    @pytest.mark.parametrize(
+        ["value", "expected"],
+        [
+            [1, TypeError],
+            [True, TypeError],
+        ],
+    )
     def test_exception_type(self, value, expected):
         with pytest.raises(expected):
             sanitize_filename(value)
