@@ -20,8 +20,9 @@ from pathvalidate import (
     sanitize_filepath,
     validate_filepath,
 )
-from pathvalidate._common import is_pathlike_obj, unprintable_ascii_chars
+from pathvalidate._common import unprintable_ascii_chars
 from pathvalidate._filepath import FilePathSanitizer, FilePathValidator
+from pathvalidate.handler import raise_error, return_null_string, return_timestamp
 
 from ._common import (
     INVALID_PATH_CHARS,
@@ -43,7 +44,11 @@ random.seed(0)
 class Test_FileSanitizer:
     @pytest.mark.parametrize(
         ["test_platform", "expected"],
-        [["windows", Platform.WINDOWS], ["linux", Platform.LINUX], ["macos", Platform.MACOS]],
+        [
+            ["windows", Platform.WINDOWS],
+            ["linux", Platform.LINUX],
+            ["macos", Platform.MACOS],
+        ],
     )
     def test_normal_platform_auto(self, monkeypatch, test_platform, expected):
         if test_platform == "windows":
@@ -53,12 +58,14 @@ class Test_FileSanitizer:
         elif test_platform == "macos":
             patch = lambda: "macos"
         else:
-            raise ValueError("unexpected test platform: {}".format(test_platform))
+            raise ValueError(f"unexpected test platform: {test_platform}")
 
         monkeypatch.setattr(m_platform, "system", patch)
 
         assert FilePathSanitizer(255, platform="auto").platform == expected
 
+
+class Test_FilePathValidator:
     @pytest.mark.parametrize(
         ["test_platform", "expected"],
         [
@@ -79,6 +86,7 @@ class Test_validate_filepath:
         "D:\\新しいフォルダー\\ユーザ属性.txt",
     ]
     WIN_VALID_PATHS = [
+        r"C:\Program Files (x86)\Microsoft",
         "D:\\Users\\\\est\\AppData\\Local\\Temp\\pytest-of-test\\pytest-0\\\\hoge.csv",
         "D:/Users/test/AppData/Local/Temp/pytest-of-test/pytest-0/test_exception__hoge_csv_heade1/hoge.csv",  # noqa
         "C:\\Users\\est\\AppData/Local\\Temp/pytest-of-test\\pytest-0/\\hoge.csv",
@@ -107,7 +115,11 @@ class Test_validate_filepath:
 
     @pytest.mark.parametrize(
         ["platform"],
-        [["linux"], ["macos"], ["posix"]],
+        [
+            ["linux"],
+            ["macos"],
+            ["posix"],
+        ],
     )
     def test_normal_only_whitespaces(self, platform):
         value = "  "
@@ -116,7 +128,10 @@ class Test_validate_filepath:
 
     @pytest.mark.parametrize(
         ["platform"],
-        [["windows"], ["universal"]],
+        [
+            ["windows"],
+            ["universal"],
+        ],
     )
     def test_abnormal_only_whitespaces(self, platform):
         value = "  "
@@ -183,34 +198,69 @@ class Test_validate_filepath:
         ],
     )
     def test_normal_max_len(self, value, platform, max_len, expected):
+        kwargs = {
+            "platform": platform,
+            "max_len": max_len,
+        }
+
         if expected is None:
-            validate_filepath(value, platform=platform, max_len=max_len)
-            assert is_valid_filepath(value, platform=platform, max_len=max_len)
+            validate_filepath(value, **kwargs)
+            assert is_valid_filepath(value, **kwargs)
             return
 
         with pytest.raises(ValidationError) as e:
-            validate_filepath(value, platform=platform, max_len=max_len)
+            validate_filepath(value, **kwargs)
         assert e.value.reason == ErrorReason.INVALID_LENGTH
 
+    @pytest.mark.parametrize(
+        ["value", "platform", "fs_encoding", "max_len", "expected"],
+        [
+            ["/tmp/" + "あ" * 83, "linux", "utf-8", 255, None],
+            ["/tmp/" + "あ" * 84, "linux", "utf-8", 255, ErrorReason.INVALID_LENGTH],
+            ["/tmp/" + "あ" * 121, "linux", "utf-16", 255, None],
+            ["/tmp/" + "あ" * 122, "linux", "utf-16", 255, ErrorReason.INVALID_LENGTH],
+        ],
+    )
+    def test_max_len_fs_encoding(self, value, platform, fs_encoding, max_len, expected):
+        kwargs = {
+            "platform": platform,
+            "max_len": max_len,
+            "fs_encoding": fs_encoding,
+        }
+
+        if expected is None:
+            validate_filepath(value, **kwargs)
+            assert is_valid_filepath(value, **kwargs)
+            return
+
+        with pytest.raises(ValidationError) as e:
+            validate_filepath(value, **kwargs)
+        assert e.value.reason == expected
+
     @pytest.mark.parametrize(
         ["value", "min_len", "max_len", "expected"],
         [
             ["valid length", 1, 255, None],
             ["minus min_len", -2, 100, None],
-            ["minus max_len", -3, -2, ValueError],
-            ["zero max_len", -2, 0, ValueError],
+            ["minus max_len", -3, -2, None],
+            ["zero max_len", -2, 0, None],
             ["eq min max", 10, 10, None],
             ["inversion", 100, 1, ValueError],
         ],
     )
     def test_minmax_len(self, value, min_len, max_len, expected):
+        kwargs = {
+            "min_len": min_len,
+            "max_len": max_len,
+        }
+
         if expected is None:
-            validate_filepath(value, min_len=min_len, max_len=max_len)
-            assert is_valid_filepath(value, min_len=min_len, max_len=max_len)
+            validate_filepath(value, **kwargs)
+            assert is_valid_filepath(value, **kwargs)
             return
 
         with pytest.raises(expected):
-            validate_filepath(value, min_len=min_len, max_len=max_len)
+            validate_filepath(value, **kwargs)
 
     @pytest.mark.parametrize(
         ["test_platform", "value", "expected"],
@@ -350,18 +400,18 @@ class Test_validate_filepath:
     @pytest.mark.parametrize(
         ["value", "platform"],
         [
-            ["/foo/abc/{}.txt".format(reserved_keyword), platform]
+            [f"/foo/abc/{reserved_keyword}.txt", platform]
             for reserved_keyword, platform in product(WIN_RESERVED_FILE_NAMES, ["linux", "macos"])
             if reserved_keyword not in [".", ".."]
         ]
         + [
-            ["{}\\{}_".format(drive, filename), platform]
+            [f"{drive}\\{filename}_", platform]
             for drive, platform, filename in product(
                 ["C:", "D:"], ["windows"], NTFS_RESERVED_FILE_NAMES
             )
         ]
         + [
-            ["{}\\abc\\{}".format(drive, filename), platform]
+            [f"{drive}\\abc\\{filename}", platform]
             for drive, platform, filename in product(
                 ["C:", "D:"], ["windows"], NTFS_RESERVED_FILE_NAMES
             )
@@ -374,26 +424,26 @@ class Test_validate_filepath:
     @pytest.mark.parametrize(
         ["value", "platform", "expected"],
         [
-            ["abc\\{}\\xyz".format(reserved_keyword), platform, ValidationError]
+            [f"abc\\{reserved_keyword}\\xyz", platform, ValidationError]
             for reserved_keyword, platform in product(
                 WIN_RESERVED_FILE_NAMES, ["windows", "universal"]
             )
             if reserved_keyword not in [".", ".."]
         ]
         + [
-            ["foo/abc/{}.txt".format(reserved_keyword), platform, ValidationError]
+            [f"foo/abc/{reserved_keyword}.txt", platform, ValidationError]
             for reserved_keyword, platform in product(WIN_RESERVED_FILE_NAMES, ["universal"])
             if reserved_keyword not in [".", ".."]
         ]
         + [
-            ["{}".format(reserved_keyword), platform, ValidationError]
+            [f"{reserved_keyword}", platform, ValidationError]
             for reserved_keyword, platform in product(
                 WIN_RESERVED_FILE_NAMES, ["windows", "universal"]
             )
             if reserved_keyword not in [".", ".."]
         ]
         + [
-            ["{}\\{}".format(drive, filename), platform, ValidationError]
+            [f"{drive}\\{filename}", platform, ValidationError]
             for drive, platform, filename in product(
                 ["C:", "D:"], ["windows"], NTFS_RESERVED_FILE_NAMES
             )
@@ -423,13 +473,18 @@ class Test_validate_filepath:
 
         assert e.value.reason == expected
         assert str(e.value) == (
-            r"invalid char found: invalids=('\r'), value='asdf\rsdf', "
-            "reason=INVALID_CHARACTER, target-platform=Windows"
+            r"[PV1100] invalid characters found: invalids=('\r'), value='asdf\rsdf', "
+            "target-platform=Windows"
         )  # noqa
 
     @pytest.mark.parametrize(
         ["value", "expected"],
-        [[None, ValueError], ["", ValidationError], [1, TypeError], [True, TypeError]],
+        [
+            [None, ValueError],
+            ["", ValidationError],
+            [1, TypeError],
+            [True, TypeError],
+        ],
     )
     def test_exception(self, value, expected):
         with pytest.raises(expected):
@@ -498,7 +553,14 @@ class Test_sanitize_filepath:
             for pair in AllPairs(
                 OrderedDict(
                     {
-                        "platform": ["posix", "linux", "macos"],
+                        "platform": [
+                            "posix",
+                            "linux",
+                            "macos",
+                            Platform.POSIX,
+                            Platform.LINUX,
+                            Platform.MACOS,
+                        ],
                         "c": INVALID_PATH_CHARS + unprintable_ascii_chars,
                         "repl": REPLACE_TEXTS,
                     }
@@ -510,7 +572,14 @@ class Test_sanitize_filepath:
             for pair in AllPairs(
                 OrderedDict(
                     {
-                        "platform": ["posix", "linux", "macos"],
+                        "platform": [
+                            "posix",
+                            "linux",
+                            "macos",
+                            Platform.POSIX,
+                            Platform.LINUX,
+                            Platform.MACOS,
+                        ],
                         "c": [":", "*", "?", '"', "<", ">", "|"],
                         "repl": REPLACE_TEXTS,
                     }
@@ -529,46 +598,46 @@ class Test_sanitize_filepath:
         ["value", "test_platform", "expected"],
         [
             [
-                "abc/{}/xyz".format(reserved_keyword),
+                f"abc/{reserved_keyword}/xyz",
                 platform,
-                "abc/{}_/xyz".format(reserved_keyword),
+                f"abc/{reserved_keyword}_/xyz",
             ]
             for reserved_keyword, platform in product(WIN_RESERVED_FILE_NAMES, ["universal"])
         ]
         + [
             [
-                "/abc/{}/xyz".format(reserved_keyword),
+                f"/abc/{reserved_keyword}/xyz",
                 platform,
-                "/abc/{}/xyz".format(reserved_keyword),
+                f"/abc/{reserved_keyword}/xyz",
             ]
             for reserved_keyword, platform in product(WIN_RESERVED_FILE_NAMES, ["linux"])
         ]
         + [
             [
-                "abc/{}.txt".format(reserved_keyword),
+                f"abc/{reserved_keyword}.txt",
                 platform,
-                "abc/{}_.txt".format(reserved_keyword),
+                f"abc/{reserved_keyword}_.txt",
             ]
             for reserved_keyword, platform in product(WIN_RESERVED_FILE_NAMES, ["universal"])
         ]
         + [
             [
-                "/abc/{}.txt".format(reserved_keyword),
+                f"/abc/{reserved_keyword}.txt",
                 platform,
-                "/abc/{}.txt".format(reserved_keyword),
+                f"/abc/{reserved_keyword}.txt",
             ]
             for reserved_keyword, platform in product(WIN_RESERVED_FILE_NAMES, ["linux"])
         ]
         + [
             [
-                "C:\\abc\\{}.txt".format(reserved_keyword),
+                f"C:\\abc\\{reserved_keyword}.txt",
                 platform,
-                "C:\\abc\\{}_.txt".format(reserved_keyword),
+                f"C:\\abc\\{reserved_keyword}_.txt",
             ]
             for reserved_keyword, platform in product(WIN_RESERVED_FILE_NAMES, ["windows"])
         ]
         + [
-            ["{}\\{}".format(drive, filename), platform, "{}\\{}_".format(drive, filename)]
+            [f"{drive}\\{filename}", platform, f"{drive}\\{filename}_"]
             for drive, platform, filename in product(
                 ["C:", "D:"], ["windows"], NTFS_RESERVED_FILE_NAMES
             )
@@ -609,7 +678,7 @@ class Test_sanitize_filepath:
     def test_normal_pathlike(self, value, replace_text, expected):
         sanitized_name = sanitize_filepath(value, replace_text)
         assert sanitized_name == expected
-        assert is_pathlike_obj(sanitized_name)
+        assert isinstance(sanitized_name, Path)
 
         validate_filepath(sanitized_name)
         assert is_valid_filepath(sanitized_name)
@@ -653,14 +722,18 @@ class Test_sanitize_filepath:
         assert sanitize_filepath(value, platform=test_platform, normalize=False) == expected
 
     @pytest.mark.parametrize(
-        ["value", "expected"],
+        ["value"],
         [
-            ["", ""],
-            [None, ""],
+            [None],
+            [""],
+            ["?"],
         ],
     )
-    def test_normal_null_values(self, value, expected):
-        assert sanitize_filepath(value) == expected
+    def test_normal_null_value_handler(self, value):
+        assert sanitize_filepath(value, null_value_handler=return_null_string) == ""
+        assert sanitize_filepath(value, null_value_handler=return_timestamp) != ""
+        with pytest.raises(ValidationError):
+            sanitize_filepath(value, null_value_handler=raise_error)
 
     @pytest.mark.parametrize(
         ["test_platform", "value", "replace_text", "expected"],
@@ -721,17 +794,46 @@ class Test_sanitize_filepath:
         ],
     )
     def test_auto_platform_linux(self, value, expected):
+        kwargs = {
+            "platform": "auto",
+            "validate_after_sanitize": True,
+        }
+
         if isinstance(expected, str):
-            sanitized = sanitize_filepath(value, platform="auto")
+            sanitized = sanitize_filepath(value, **kwargs)
             assert is_valid_filepath(sanitized, platform="auto")
-        else:
-            with pytest.raises(expected):
-                sanitize_filepath(value, platform="auto")
+            return
+
+        with pytest.raises(expected) as e:
+            sanitize_filepath(value, **kwargs)
+        assert e.value.reason == ErrorReason.MALFORMED_ABS_PATH
+
+    @pytest.mark.parametrize(
+        ["platform", "value"],
+        [
+            ["windows", "CON \r"],
+        ],
+    )
+    def test_exception_invalid_after_sanitize(self, platform, value):
+        print(
+            "'{}'".format(
+                sanitize_filepath(value, platform=platform, validate_after_sanitize=False)
+            ),
+            file=sys.stderr,
+        )
+        with pytest.raises(ValidationError) as e:
+            sanitize_filepath(value, platform=platform, validate_after_sanitize=True)
+        assert e.value.reason == ErrorReason.INVALID_AFTER_SANITIZE
 
     @pytest.mark.skipif(sys.version_info < (3, 6), reason="requires python3.6 or higher")
     @pytest.mark.parametrize(
         ["value", "expected"],
-        [[1, TypeError], [True, TypeError], [nan, TypeError], [inf, TypeError]],
+        [
+            [1, TypeError],
+            [True, TypeError],
+            [nan, TypeError],
+            [inf, TypeError],
+        ],
     )
     def test_exception_type(self, value, expected):
         with pytest.raises(expected):
